package main;

import entity.Region;
import entity.CustomerOrder;
import entity.CustomerOrder_;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

public class OrderService {


    private static final String CONNECTION_URL = "jdbc:hsqldb:mem:testDB";
    private static final String USER ="SA";
    private static final String PASSWORD = "";
    private static final boolean NULL_WORKAROUND = false;

    private EntityManager entityManager;

    public OrderService(EntityManager entityManager) {
        this.entityManager = entityManager;
    }
    
    
    public List <CustomerOrder> getAll(){
        
        TypedQuery<CustomerOrder> query = entityManager.createNamedQuery(CustomerOrder.GET_ALL, CustomerOrder.class);
        List <CustomerOrder> resultList = query.getResultList();
        return resultList;
    }

    public List <CustomerOrder> getByNameNull(){
        
        TypedQuery<CustomerOrder> query = entityManager.createNamedQuery(CustomerOrder.GET_BY_NAME, CustomerOrder.class);
        query.setParameter("name", null);
        List <CustomerOrder> resultList = query.getResultList();
        return resultList;
    }
    
    public List <CustomerOrder> getByRegionIdNull(){
        
        TypedQuery<CustomerOrder> query = entityManager.createNamedQuery(CustomerOrder.GET_EQUAL_ID, CustomerOrder.class);
        query.setParameter("regionId", null);
        List <CustomerOrder> resultList = query.getResultList();
        return resultList;
        
    }
    
    public List <CustomerOrder> getByRegionNull(){
        
        TypedQuery<CustomerOrder> query = entityManager.createNamedQuery(CustomerOrder.GET_EQUAL_OBJECT, CustomerOrder.class);
        query.setParameter("region", null);
        List <CustomerOrder> resultList = query.getResultList();
        return resultList;
        
    }
    
    
    public List <CustomerOrder> getByRegionIdNullCriteriaQuery(){
        
        Integer regionId = null;
        CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
        CriteriaQuery<CustomerOrder> criteriaQuery = criteriaBuilder.createQuery(CustomerOrder.class);
        Root<CustomerOrder> customerOrderRoot = criteriaQuery.from(CustomerOrder.class);
        // Apparently passing null into this does not work:
        // Predicate condition = criteriaBuilder.equal(customerOrderRoot.get(CustomerOrder_.regionId), null);
        Predicate condition;
        if(regionId == null){
            condition = criteriaBuilder.isNull(customerOrderRoot.get(CustomerOrder_.regionId));
        }else{
            condition = criteriaBuilder.equal(customerOrderRoot.get(CustomerOrder_.regionId), regionId);
        }
        criteriaQuery.where(condition);
        TypedQuery<CustomerOrder> typedQuery = entityManager.createQuery(criteriaQuery); 
        List<CustomerOrder> resultList = typedQuery.getResultList();        
        return resultList;
        
    }
    
    public List <CustomerOrder> getByRegionNullCriteriaQuery(){
        
        Region region = null;
        CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
        CriteriaQuery<CustomerOrder> criteriaQuery = criteriaBuilder.createQuery(CustomerOrder.class);
        Root<CustomerOrder> customerOrderRoot = criteriaQuery.from(CustomerOrder.class);
        // Apparently passing null into this does not work:
        // Predicate condition = criteriaBuilder.equal(customerOrderRoot.get(CustomerOrder_.region), null);
        Predicate condition;
        if(region == null){
            condition = criteriaBuilder.isNull(customerOrderRoot.get(CustomerOrder_.region));
        }else{
            condition = criteriaBuilder.equal(customerOrderRoot.get(CustomerOrder_.region), region);
        }
        criteriaQuery.where(condition);
        TypedQuery<CustomerOrder> typedQuery = entityManager.createQuery(criteriaQuery); 
        List<CustomerOrder> resultList = typedQuery.getResultList();        
        return resultList;
        
    }
    public List<Integer> getOrderIdsJdbcPrepared(){
        Connection conn = null;
        try{
            conn = getConnection();
            String sql = "SELECT ID, NAME, region_id FROM CUSTOMERORDER WHERE ((? IS NULL) OR (region_id = ?))";
            PreparedStatement pStmt = conn.prepareStatement(sql);
            Integer regionId = null;
            if(regionId == null && NULL_WORKAROUND){
                pStmt.setNull(1, Types.INTEGER);
                pStmt.setNull(2, Types.INTEGER);
            }else{
                pStmt.setObject(1, regionId);
                pStmt.setObject(2, regionId);
            }
            ResultSet result = pStmt.executeQuery();
            List<Integer> resultList = new ArrayList<Integer>();
            while(result.next()){
                int id = result.getInt(1);
                String name = result.getString(2);
                Integer regionIdResult = (Integer)result.getObject(3);
                resultList.add(Integer.valueOf(id));
            }
            return resultList;
        }catch(SQLException ex){
            throw new RuntimeException(ex);
        }
        finally{
            if(conn != null){
                try{
                    conn.close();
                }catch(Exception ignore){}
            }
        }
            
    }
    
    public List<Integer> getOrderIdsJdbcStatic(){
        Connection conn = null;
        try{
            conn = getConnection();
            Statement statement = conn.createStatement();
            String sql = "SELECT ID, NAME, region_id FROM CUSTOMERORDER WHERE ((NULL IS NULL) OR (region_id = NULL))";
            ResultSet result = statement.executeQuery(sql);
            List<Integer> resultList = new ArrayList<Integer>();
            while(result.next()){
                int id = result.getInt(1);
                String name = result.getString(2);
                Integer regionIdResult = (Integer)result.getObject(3);
                resultList.add(Integer.valueOf(id));
            }
            return resultList;
        }catch(SQLException ex){
            throw new RuntimeException(ex);
        }
        finally{
            if(conn != null){
                try{
                    conn.close();
                }catch(Exception ignore){}
            }
        }
            
    }
    
    private Connection getConnection() throws SQLException{
        DriverManager.registerDriver(new org.hsqldb.jdbc.JDBCDriver());
        Connection conn = DriverManager.getConnection(CONNECTION_URL, USER, PASSWORD);
        return conn;
    }


}
