package main;

import entity.CustomerOrder;
import entity.Region;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import org.junit.*;
import static org.junit.Assert.*;


/**
 * Please run this class with CREATE_NEW = true once so that the database gets some content.
 * All tests should pass.
 */
public class OrderServiceTest {
    
    public static final Integer REGION_ID = 1;
    public static final Integer ORDER_ID = 1;
    private static final boolean CREATE_NEW = true;
    
    
    private static EntityManager entityManager;
    private static EntityTransaction trans;
    private static EntityManagerFactory emf;
    
    private static OrderService instance;

    @BeforeClass
    public static void setUpClass() throws Exception {

        if(CREATE_NEW){
            emf = Persistence.createEntityManagerFactory("DropAndCreatePU");
            entityManager = emf.createEntityManager();
            trans = entityManager.getTransaction();
            trans.begin();
            createData();
            trans.commit();
            entityManager.close();
        }else{
            emf = Persistence.createEntityManagerFactory("UpdatePU");
        }

        entityManager = emf.createEntityManager();
        
    }

    @AfterClass
    public static void tearDownClass() throws Exception {

        entityManager.close();
        emf.close();
        
    }
    
    @Before
    public void setUp() {
        
        trans = entityManager.getTransaction();
        trans.begin();
        instance = new OrderService(entityManager);

    }
    
    @After
    public void tearDown() {
        entityManager.flush();
        trans.rollback();
    }

    @Test
    public void testGetAll() {
        System.out.println("getAll");
        List result = instance.getAll();
        assertEquals(1, result.size());
    }

    @Test
    public void testGetByNameNull() {
        System.out.println("getByNameNull");
        List result = instance.getByNameNull();
        assertEquals(1, result.size());
    }

    @Test
    public void testGetByRegionIdNull() {
        System.out.println("getByRegionIdNull");
        List result = instance.getByRegionIdNull();
        assertEquals(1, result.size());
    }

    @Test
    public void testGetByRegionNull() {
        System.out.println("getByRegionNull");
        List result = instance.getByRegionNull();
        assertEquals(1, result.size());
    }

    @Test
    public void testGetByRegionIdNullCriteriaQuery() {
        System.out.println("getByRegionIdNullCriteriaQuery");
        List result = instance.getByRegionIdNullCriteriaQuery();
        assertEquals(1, result.size());
    }

    @Test
    public void testGetByRegionNullCriteriaQuery() {
        System.out.println("getByRegionNullCriteriaQuery");
        List result = instance.getByRegionNullCriteriaQuery();
        assertEquals(1, result.size());
    }
    
    @Test
    public void testGetOrderIdsJdbcPrepared() {
        System.out.println("getOrderIdsJdbcPrepared");
        List<Integer> resultList = instance.getOrderIdsJdbcPrepared();
        assertEquals(1, resultList.size());
    }
    
    @Test
    public void testGetOrderIdsJdbcStatic() {
        System.out.println("getOrderIdsJdbcStatic");
        List<Integer> resultList = instance.getOrderIdsJdbcStatic();
        assertEquals(1, resultList.size());
    }
    
    
    private static void createData(){

        Region region = new Region(REGION_ID);
        entityManager.persist(region);
        CustomerOrder order = new CustomerOrder(ORDER_ID);
        entityManager.persist(order);

    }
    
}
